@@ -43,6 +43,8 @@
 #include <openssl/err.h>
 #include <QSqlQuery>
 #include <QSqlError>
+#include <openssl/rand.h>
+
 static const char *KJSonEncryptedMessageEkElement = "ek" ;
 static const char *KJSonEncryptedMessageIvElement = "iv" ;
 
@@ -74,20 +76,28 @@ Hash ContentEncryptionModel::generateKeyPair() {
 
     LOG_STR("ContentEncryptionModel::generateKeyPair in") ;
 
-    EVP_PKEY * pkey;
+    EVP_PKEY * pkey (NULL);
     pkey = EVP_PKEY_new();
-    RSA * rsa;
-    if ((rsa = RSA_generate_key(
-                   2048,   /* number of bits for the key - 2048 is a sensible value */
-                   RSA_F4, /* exponent - RSA_F4 is defined as 0x10001L */
-                   NULL,   /* callback - can be NULL if we aren't displaying progress */
-                   NULL    /* callback argument - not needed in this case */
-               ) ) == NULL ) {
+    RSA * rsa (NULL) ;
+    BIGNUM          *bne ( NULL );
+    int ret = 0 ; 
+    bne = BN_new();
+    unsigned long   e = RSA_F4;
+    ret = BN_set_word(bne,e);
+    rsa = RSA_new() ; 
+
+    if ((ret = RSA_generate_key_ex(rsa,
+                                   2048,   
+                                   bne,
+                                   NULL    
+             ) ) != 1 ) {
         QString errmsg(tr("SSL key generation went wrong, calling exit..")) ;
         emit error(MController::ContentEncryptionError, errmsg) ;
+        BN_free(bne) ; 
+        RSA_free(rsa) ; 
         return retval ;
     }
-    EVP_PKEY_assign_RSA(pkey, rsa);
+    EVP_PKEY_assign_RSA(pkey, rsa); // after this point rsa can't be free'd
     X509 * x509;
     x509 = X509_new();
     if ( x509 == NULL ) {
@@ -305,7 +315,7 @@ int ContentEncryptionModel::sign(const Hash& aSigningKey,
                                  const QByteArray* aOptionalMetadata) {
     int retval = -1 ;
     LOG_STR2("ContentEncryptionModel::sign in, bytesToSign=%d", aData.length());
-    EVP_MD_CTX     md_ctx;
+    EVP_MD_CTX     *md_ctx = EVP_MD_CTX_create();
     unsigned sig_len;
     unsigned char sig_buf [4096];
     int err ;
@@ -318,15 +328,15 @@ int ContentEncryptionModel::sign(const Hash& aSigningKey,
             // taken from openssl examples by Eric Young & Sampo Kellomaki
             /* Do the signature */
 
-            EVP_SignInit   (&md_ctx, EVP_sha1());
-            EVP_SignUpdate (&md_ctx, aData.data(), aData.length());
+            EVP_SignInit   (md_ctx, EVP_sha1());
+            EVP_SignUpdate (md_ctx, aData.data(), aData.length());
             if ( aOptionalMetadata ) {
-                EVP_SignUpdate (&md_ctx,
+                EVP_SignUpdate (md_ctx,
                                 aOptionalMetadata->data(),
                                 aOptionalMetadata->length());
             }
             sig_len = sizeof(sig_buf);
-            err = EVP_SignFinal (&md_ctx, sig_buf, &sig_len, keyToUse);
+            err = EVP_SignFinal (md_ctx, sig_buf, &sig_len, keyToUse);
 
             if (err != 1) {
                 QString errmsg(ERR_reason_error_string(ERR_get_error())) ;
@@ -341,6 +351,7 @@ int ContentEncryptionModel::sign(const Hash& aSigningKey,
             EVP_PKEY_free (keyToUse);
         }
     }
+    EVP_MD_CTX_destroy(md_ctx);
     return retval ;
 }
 
@@ -383,18 +394,18 @@ bool  ContentEncryptionModel::doVerify(const QByteArray& aPemBytesOfSigningKey,
         EVP_PKEY *keyToUse = PublicKeyFromPem(aPemBytesOfSigningKey)  ;
         if ( keyToUse ) {
 
-            EVP_MD_CTX     md_ctx;
+            EVP_MD_CTX     *md_ctx ( EVP_MD_CTX_create() ) ;
             int err ;
             /* Verify the signature */
 
-            EVP_VerifyInit   (&md_ctx, EVP_sha1());
-            EVP_VerifyUpdate (&md_ctx, aDataToVerify.data(), aDataToVerify.length());
+            EVP_VerifyInit   (md_ctx, EVP_sha1());
+            EVP_VerifyUpdate (md_ctx, aDataToVerify.data(), aDataToVerify.length());
             if ( aOptionalMetadata ) {
-                EVP_VerifyUpdate (&md_ctx,
+                EVP_VerifyUpdate (md_ctx,
                                   aOptionalMetadata->data(),
                                   aOptionalMetadata->length());
             }
-            err = EVP_VerifyFinal (&md_ctx,
+            err = EVP_VerifyFinal (md_ctx,
                                    (const unsigned char *) aSignatureToVerify.data(),
                                    aSignatureToVerify.length(),
                                    keyToUse);
@@ -408,6 +419,7 @@ bool  ContentEncryptionModel::doVerify(const QByteArray& aPemBytesOfSigningKey,
                 retval = true ; // ahem
             }
             EVP_PKEY_free (keyToUse);
+            EVP_MD_CTX_destroy(md_ctx);
         }
     }
     return retval ;
@@ -419,7 +431,7 @@ bool ContentEncryptionModel::encrypt(const QList<Hash> aRecipients,
     // significant parts of code here is stolen from openssl wiki.
     bool retval (false) ;
     // for SealInit
-    EVP_CIPHER_CTX ctx;
+    EVP_CIPHER_CTX *ctx (EVP_CIPHER_CTX_new());
     unsigned char **ek = NULL ;
     int *ekl(NULL);
     unsigned char *iv ( (unsigned char *) malloc ( EVP_CIPHER_iv_length(EVP_aes_256_cbc() ) ) );
@@ -428,7 +440,7 @@ bool ContentEncryptionModel::encrypt(const QList<Hash> aRecipients,
     QMap<QString,QVariant> resultJSon ;
 
     if ( aRecipients.size() > 0 && aPlainText.length() > 0 && iv ) {
-        EVP_CIPHER_CTX_init(&ctx);
+        EVP_CIPHER_CTX_init(ctx);
         pubk =( EVP_PKEY ** )  malloc (sizeof(EVP_PKEY *)*aRecipients.size()) ;
 
         if ( pubk ) {
@@ -457,7 +469,7 @@ bool ContentEncryptionModel::encrypt(const QList<Hash> aRecipients,
                         }
                     }
                     if ( alloc_failure == false ) {
-                        if (!EVP_SealInit(&ctx, EVP_aes_256_cbc(), ek, ekl, iv, pubk, nr_of_valid_pubkeys)) {
+                        if (!EVP_SealInit(ctx, EVP_aes_256_cbc(), ek, ekl, iv, pubk, nr_of_valid_pubkeys)) {
                             QString errmsg(ERR_reason_error_string(ERR_get_error())) ;
                             emit error(MController::ContentEncryptionError, errmsg) ;
                         } else {
@@ -494,7 +506,7 @@ bool ContentEncryptionModel::encrypt(const QList<Hash> aRecipients,
                             while ( pos < aPlainText.size() ) {
                                 QByteArray fourKiloByte ( aPlainText.mid(pos,FOUR_KBYTES)) ;
                                 LOG_STR2("FourKByteLen: %d",fourKiloByte.length());
-                                if (!EVP_SealUpdate(&ctx,
+                                if (!EVP_SealUpdate(ctx,
                                                     buffer_out,
                                                     &len_out,
                                                     (const unsigned char*)fourKiloByte.data(),
@@ -513,7 +525,7 @@ bool ContentEncryptionModel::encrypt(const QList<Hash> aRecipients,
                             }
 
                             if ( failure == false ) {
-                                if (!EVP_SealFinal(&ctx, buffer_out, &len_out)) {
+                                if (!EVP_SealFinal(ctx, buffer_out, &len_out)) {
                                     QString errmsg(ERR_reason_error_string(ERR_get_error())) ;
                                     emit error(MController::ContentEncryptionError, errmsg) ;
                                 } else  {
@@ -540,7 +552,8 @@ bool ContentEncryptionModel::encrypt(const QList<Hash> aRecipients,
             free(pubk) ;
         } // if pubk was mallocated ok
         free(iv) ;
-        EVP_CIPHER_CTX_cleanup(&ctx);
+        EVP_CIPHER_CTX_cleanup(ctx);
+        EVP_CIPHER_CTX_free(ctx);
     }
 
     LOG_STR2("Res = %s", qPrintable(QString(aResultingCipherText)));
@@ -576,16 +589,16 @@ bool ContentEncryptionModel::decrypt(const QByteArray& aCipherText,
                     if ( myPrivateKey ) {
                         pemBytes.clear() ;
                         // de-cryption context
-                        EVP_CIPHER_CTX ctx;
+                        EVP_CIPHER_CTX *ctx ( EVP_CIPHER_CTX_new() );
                         // try out each key in array that is in the encrypted document,
                         // if one of those would happen to be for ours.. ..
                         bool keyFound ( false ) ;
                         foreach( QVariant keyVariant, encryptionKeyList) {
                             LOG_STR("Trying private key..") ;
                             const QByteArray key ( QByteArray::fromBase64 (keyVariant.toByteArray()) ) ;
-                            EVP_CIPHER_CTX_init(&ctx);
+                            EVP_CIPHER_CTX_init(ctx);
 
-                            if (EVP_OpenInit(&ctx,
+                            if (EVP_OpenInit(ctx,
                                              EVP_aes_256_cbc(),
                                              (const unsigned char *)(key.constData()),
                                              key.length(),
@@ -596,8 +609,8 @@ bool ContentEncryptionModel::decrypt(const QByteArray& aCipherText,
                                 break ; // out from foreach-loop
                             } else {
                                 // key not found, cleanup context and try again with next key:
-                                EVP_CIPHER_CTX_cleanup(&ctx);
-                                EVP_CIPHER_CTX_init(&ctx);
+                                EVP_CIPHER_CTX_cleanup(ctx);
+                                EVP_CIPHER_CTX_init(ctx);
                             }
                         } // foreach key
                         if ( keyFound ) {
@@ -606,7 +619,7 @@ bool ContentEncryptionModel::decrypt(const QByteArray& aCipherText,
                             int len_out ( 0 ) ;
                             unsigned char* plainTextPtr = (unsigned char*)malloc(aCipherText.length()-metaDataLen) ;
                             if ( plainTextPtr ) {
-                                if (!EVP_OpenUpdate(&ctx,
+                                if (!EVP_OpenUpdate(ctx,
                                                     plainTextPtr,
                                                     &len_out,
                                                     (const unsigned char *)(aCipherText.mid(sizeof(quint32)+metaDataLen).constData()),
@@ -619,7 +632,7 @@ bool ContentEncryptionModel::decrypt(const QByteArray& aCipherText,
                                     if ( len_out > 0 ) {
                                         aResultingPlainText.append((const char *)plainTextPtr,len_out) ;
                                     }
-                                    if ( !EVP_OpenFinal(&ctx, plainTextPtr, &len_out) ) {
+                                    if ( !EVP_OpenFinal(ctx, plainTextPtr, &len_out) ) {
                                         QString errmsg(ERR_reason_error_string(ERR_get_error())) ;
                                         emit error(MController::ContentEncryptionError, errmsg) ;
                                     }  else {
@@ -640,7 +653,8 @@ bool ContentEncryptionModel::decrypt(const QByteArray& aCipherText,
                             LOG_STR("Decrypt: key not found") ;
                             retval = false ;
                         }
-                        EVP_CIPHER_CTX_cleanup(&ctx);
+                        EVP_CIPHER_CTX_cleanup(ctx);
+                        EVP_CIPHER_CTX_free(ctx);
                         EVP_PKEY_free(myPrivateKey) ;
                     }
                 }
@@ -898,3 +912,11 @@ Hash ContentEncryptionModel::hashOfPublicKey(const QByteArray& aPemBytes) {
     return retval ;
 }
 
+
+QByteArray ContentEncryptionModel::randomBytes(int aNumberOfBytes) {
+    unsigned char* bytes = new unsigned char[aNumberOfBytes+1] ; 
+    RAND_bytes(bytes, aNumberOfBytes) ; 
+    QByteArray retval((const char *)bytes,aNumberOfBytes) ;
+    delete bytes ;
+    return retval ; 
+}
