@@ -26,6 +26,7 @@
 #include <QHostAddress> // for Q_IPV6ADDR
 #include <QApplication>
 #include "util/catranslator.h"
+#include <QRegExpValidator>
 
 QApplication* app ; /**< The qt application, we need to have 1 instance */
 Controller* c ; /** Application controller, here as static for signal handlers */
@@ -68,6 +69,7 @@ void sigUSR2handler(int) {
     LOG_STR("SIGUSR2 trapped..") ;
     if ( c != NULL ) {
         c->showUI() ;
+        c->checkForSharedMemoryContents() ;
     }
 }
 #endif
@@ -86,6 +88,26 @@ int main(int argc, char *argv[]) {
         QLOG_STR( "WSAStartup() success");
     }
 #endif
+#if !(defined(WIN32)||defined(Q_OS_OSX))
+#if QT_VERSION >= 0x050000
+    QString platform ( QGuiApplication::platformName() ) ;
+    bool have_xcb ( platform.compare("xcb") == 0 || platform.length()==0) ;
+    QLOG_STR("Platform: " + QGuiApplication::platformName() +
+             " len = " +
+             QString::number(platform.length())) ;
+    bool have_display ( getenv("DISPLAY") != NULL  ) ;
+    bool have_wayland ( getenv("WAYLAND_DISPLAY") != NULL ) ;
+    if ( have_xcb && (
+                have_display==false &&
+                have_wayland==false ) ) {
+        // so, we have "xcb" that is normal linux. and we have no $DISPLAY
+        // nor $WAYLAND_DISPLAY -> this spells some problems..
+        fprintf(stderr,"No $DISPLAY/WAYLAND_DISPLAY environment variable set, cant continue\n") ;
+        return 0 ;
+    }
+#endif // qt version
+#endif // check of $DISPLAY or $WAYLAND_DISPLAY
+
     app = new QApplication (argc, argv);
 
     CATranslator caTranslator;
@@ -129,7 +151,27 @@ int main(int argc, char *argv[]) {
     signal(SIGUSR1,sigUSR1handler);
     signal(SIGUSR2,sigUSR2handler);
 #endif
-    int retval = app->exec() ;
+
+    // check for possible command line arguments relevant to
+    // us:
+    QRegExp rx("^(caprofile|caad|cacomment|cablob)://[a-fA-F0-9]{40}/{0,1}$");
+    QRegExpValidator validator (rx);
+    for ( int i = 1 ; i < argc ; i++ ) {
+        QString argumentCandidate(argv[i] );
+        int position ( 0 ) ;
+        if ( validator.validate(argumentCandidate,position) == QValidator::Acceptable ) {
+            QUrl commandLineUrl ( argumentCandidate ) ;
+            QLOG_STR("scheme " + commandLineUrl.scheme() ) ;
+            QLOG_STR("host " + commandLineUrl.host() ) ;
+            c->addObjectToOpen(commandLineUrl) ;
+            break ; // out of the loop, process only one
+        }
+    }
+    int retval ( 0 ) ;
+    if ( c->init() ) { // 2nd stage of constructor
+        retval = app->exec() ;
+    }
+    QLOG_STR("deleting controller") ;
     delete c ;
     delete app ;
     return retval ;
