@@ -1,21 +1,21 @@
 /*     -*-C++-*- -*-coding: utf-8-unix;-*-
-       Classified Ads is Copyright (c) Antti Järvinen 2013.
+  Classified Ads is Copyright (c) Antti Järvinen 2013.
 
-       This file is part of Classified Ads.
+  This file is part of Classified Ads.
 
-    Classified Ads is free software; you can redistribute it and/or
-    modify it under the terms of the GNU Lesser General Public
-    License as published by the Free Software Foundation; either
-    version 2.1 of the License, or (at your option) any later version.
+  Classified Ads is free software; you can redistribute it and/or
+  modify it under the terms of the GNU Lesser General Public
+  License as published by the Free Software Foundation; either
+  version 2.1 of the License, or (at your option) any later version.
 
-    Classified Ads is distributed in the hope that it will be useful,
-       but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    Lesser General Public License for more details.
+  Classified Ads is distributed in the hope that it will be useful,
+  but WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+  Lesser General Public License for more details.
 
-    You should have received a copy of the GNU Lesser General Public
-    License along with Classified Ads; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+  You should have received a copy of the GNU Lesser General Public
+  License along with Classified Ads; if not, write to the Free Software
+  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */
 
 #ifdef WIN32
@@ -37,6 +37,9 @@
 #include "../datamodel/binaryfilemodel.h"
 #include "../datamodel/privmsgmodel.h"
 #include "../datamodel/profilecommentmodel.h"
+#include <QWaitCondition>
+#include <QMutex>
+#include <QSslCipher>
 
 /** nat subnet. see similar variable in node.cpp */
 static const QPair<QHostAddress, int> normalNats1 ( QHostAddress::parseSubnet("10.0.0.0/8") );
@@ -73,7 +76,8 @@ Connection::Connection (const QHostAddress& aAddr,
       iSocketOpenTime(QDateTime::currentDateTimeUtc().toTime_t()),
       iBytesIn(0),
       iBytesOut(0),
-      iPeerAddress(aAddr) {
+      iPeerAddress(aAddr),
+      iSleepBetweenSendOperations(500)  {
     LOG_STR(QString("0x%1 ").arg((qulonglong)this, 8) +"Connection::Connection outgoing") ;
 }
 
@@ -99,7 +103,8 @@ Connection::Connection(int aSocketDescriptor,
       iTimeOfBucketFill(QDateTime::currentDateTimeUtc().toTime_t() ),
       iSocketOpenTime(QDateTime::currentDateTimeUtc().toTime_t()),
       iBytesIn(0),
-      iBytesOut(0)  {
+      iBytesOut(0),
+      iSleepBetweenSendOperations(500)  {
     LOG_STR(QString("0x%1 ").arg((qulonglong)this, 8) +"Connection::Connection incoming") ;
 }
 
@@ -141,6 +146,10 @@ void Connection::run() {
     } else {
         runForOutgoingConnections() ;
     }
+    if ( iConnectionState == Initial ||
+            iConnectionState == Open ) {
+        iConnectionState = Closing ;
+    }
     iObserver.connectionClosed(this) ; // will call datamodel.lock() inside
     LOG_STR(QString("0x%1 ").arg((qulonglong)this, 8) +"Connection::run out") ;
     emit finished() ;
@@ -157,8 +166,9 @@ void Connection::runForIncomingConnections() {
     QSslSocket sock ;
     iSocket = &sock ; // when this method goes out of
     // scope, iSocket must be set to NULL
+
     if (iSocket->setSocketDescriptor(iSocketDescriptor)) {
-        connect(iSocket, SIGNAL(encrypted()), this, SLOT(socketReady()));
+        setupSocketSignals() ;
         // the idea here with model.lock is that
         // this class instance here is owned by datamodel
         // and datamodel may frobnicate our internals so
@@ -182,20 +192,14 @@ void Connection::runForIncomingConnections() {
         return ;
     }
     iSocket->setSocketOption(QAbstractSocket::LowDelayOption,1) ; // immediate send
-    if ( iSocket->waitForEncrypted(10*1000) == false ) {
-        LOG_STR(QString("0x%1 ").arg((qulonglong)this, 8) +"socket was not encrypted in 10 seconds -> out") ;
-        iSocket->abort() ;
-        return ;
-    }
-
-    iPeerAddress = iSocket->peerAddress() ;
-    if ( iPeerAddress == iSocket->localAddress() ) {
-        QLOG_STR(QString("0x%1 ").arg((qulonglong)this, 8) + "Connected to self, addr = " + iSocket->peerAddress().toString()) ;
-        // oh great, broadcast?
-        // connected to self
-        iNeedsToRun = false ;
-        iSocket->close() ;
-        return ;
+    for ( int i = 0 ; iConnectionState == Initial ; i++ ) {
+        msleep(1000) ;
+        QCoreApplication::processEvents(QEventLoop::ExcludeUserInputEvents) ;
+        if ( i > 10 ) { // 10 seconds passed and connection state not changed
+            iSocket->abort() ;
+            iNeedsToRun = false ;
+            return ;
+        }
     }
 
     // ok, obviously we can receive, mark that down for future
@@ -206,6 +210,7 @@ void Connection::runForIncomingConnections() {
     // ok, we came here, we have SSL handshake done ; lets
     // instantiate some data structures..
     Hash fingerPrintOfPeer (iSocket->peerCertificate()) ;
+    iPeerHash = fingerPrintOfPeer ;
     // it also may be so that connection was attempted to a particular
     // node - is it possible that we managed to connect other node
     // than that?
@@ -242,15 +247,35 @@ void Connection::runForIncomingConnections() {
     if ( iNeedsToRun == false && iSocket &&  iSocket->state() == QAbstractSocket::ConnectedState ) {
         iSocket->close() ;
     }
-    iSocket->waitForDisconnected(10*1000) ;
+    if ( iSocket ) {
+        iSocket->waitForDisconnected(10*1000) ;
+    }
     LOG_STR(QString("0x%1 ").arg((qulonglong)this, 8) +"Connection::run(incoming) out") ;
 }
 
 void Connection::socketReady() {
-    LOG_STR(QString("0x%1 ").arg((qulonglong)this, 8) +"Connection::socketReady") ;
+    LOG_STR(QString("0x%1 ").arg((qulonglong)this, 8) +
+            "Connection::socketReady auth " +
+            iSocket->sessionCipher().authenticationMethod() +
+            " keyEx " +
+            iSocket->sessionCipher().keyExchangeMethod() +
+            " encryption " +
+            iSocket->sessionCipher().encryptionMethod() +
+            " bits " +
+            QString::number(iSocket->sessionCipher().usedBits())) ;
     iModel.lock() ;
     iConnectionState = Open ;
     iModel.connectionStateChanged(iConnectionState, this) ;
+    iPeerAddress = iSocket->peerAddress() ;
+    iPeerHash = Hash(   iSocket->peerCertificate() );
+    if ( iPeerAddress == iSocket->localAddress() ) {
+        QLOG_STR(QString("0x%1 ").arg((qulonglong)this, 8) + "Connected to self, addr = " + iSocket->peerAddress().toString()) ;
+        // oh great, broadcast?
+        // connected to self
+        iNeedsToRun = false ;
+        iSocket->close() ;
+        return ;
+    }
     iModel.unlock() ;
 }
 
@@ -263,7 +288,7 @@ void Connection::runForOutgoingConnections () {
     QSslSocket sock ;
     iSocket = &sock ;
 
-    connect(iSocket, SIGNAL(encrypted()), this, SLOT(socketReady()));
+    setupSocketSignals() ;
     // the idea here with model.lock is that
     // this class instance here is owned by datamodel
     // and datamodel may frobnicate our internals so
@@ -284,10 +309,17 @@ void Connection::runForOutgoingConnections () {
     QLOG_STR(QString("0x%1 ").arg((qulonglong)this, 8) +"Connecting to " + iAddrToConnect.toString() + " port " + QString::number(iPortToConnect) ) ;
     iSocket->connectToHostEncrypted(iAddrToConnect.toString(), iPortToConnect);
 
-    if(!iSocket->waitForEncrypted(10*1000)) {
-        LOG_STR(QString("0x%1 ").arg((qulonglong)this, 8) +"connectToHostEncrypted failed") ;
-        iSocket->abort() ;
-        return ;
+    for ( int i = 0 ; iConnectionState == Initial ; i++ ) {
+        QCoreApplication::processEvents(QEventLoop::ExcludeUserInputEvents) ;
+        msleep(1000) ;
+        if ( i > 10 ) { // 10 seconds passed and connection state not changed
+            iSocket->abort() ;
+            iNeedsToRun = false ;
+            QLOG_STR("Connection attempt to " + iAddrToConnect.toString() + " timed out") ;
+            QLOG_STR("Connection attempt to " + iFpOfNodeWeTrying.toString() + " timed out") ;
+            emit connectionAttemptFailed(iFpOfNodeWeTrying) ; // then don't try again immediately
+            return ;
+        }
     }
 
     if ( iSocket->peerAddress() == iSocket->localAddress() ) {
@@ -300,7 +332,7 @@ void Connection::runForOutgoingConnections () {
     }
 
     Hash fingerPrintOfPeer (iSocket->peerCertificate()) ;
-
+    iPeerHash = fingerPrintOfPeer ;
     // it also may be so that connection was attempted to a particular
     // node - is it possible that we managed to connect other node
     // than that?
@@ -322,10 +354,6 @@ void Connection::runForOutgoingConnections () {
     }
     iNodeOfConnectedPeer= new Node(fingerPrintOfPeer, -1) ;
 
-    // ok, we came here, we have SSL handshake done ; lets
-    // instantiate some data structures..
-    iNodeOfConnectedPeer= new Node(Hash(fingerPrintOfPeer),
-                                   -1) ;
     // we just managed to connect, mark that thing down for
     // future advertisement
     iModel.lock() ;
@@ -337,7 +365,7 @@ void Connection::runForOutgoingConnections () {
     if ( iNeedsToRun == false && iSocket &&  iSocket->state() == QAbstractSocket::ConnectedState ) {
         iSocket->close() ;
     }
-    if ( iNeedsToRun && iSocket ) {
+    if ( iSocket ) {
         iSocket->waitForDisconnected(30*1000) ;
     }
     iSocket = NULL ;
@@ -347,52 +375,65 @@ void Connection::runForOutgoingConnections () {
 
 // this is called from run() method when there is data available
 void Connection::performRead() {
-    if ( iBytesExpectedInPacketBeingRead == 0 ) {
-        // we're not reading a packet yet, read packet
-        quint32 bytesExpectedNetworkByteOrder = 0 ;
-        if ( iSocket->read((char *)&bytesExpectedNetworkByteOrder,
-                           sizeof(quint32)) != sizeof(quint32)) {
-            iNeedsToRun = false ;
-            iModel.lock() ;
-            iModel.connectionStateChanged(Connection::Closing, this) ;
-            iModel.unlock() ;
-            iSocket->abort() ;
-            return ;
-        } else {
-            iBytesIn += sizeof(quint32) ;
+    quint64 bytesAvail (0) ;
+
+    while ( ( bytesAvail = iSocket->bytesAvailable() ) > 0 ) {
+        if ( iBytesExpectedInPacketBeingRead == 0 &&
+                bytesAvail < sizeof(quint32) ) {
+            msleep(50) ; // size not yet received, wait for 50 ms
         }
-        iBytesExpectedInPacketBeingRead = ntohl ( bytesExpectedNetworkByteOrder);
-        QLOG_STR(QString("0x%1 ").arg((qulonglong)this, 8) +"Connection::performRead expecting bytes= "+ QString::number( iBytesExpectedInPacketBeingRead)) ;
-    }
-    quint64 bytesAvail = 0 ;
-    while ( iBytesExpectedInPacketBeingRead > 0 &&
-            iBytesExpectedInPacketBeingRead > (quint64)(iBytesRead->size()) &&
-            (bytesAvail = iSocket->bytesAvailable()) > 0 ) {
-        if ( bytesAvail > iBytesExpectedInPacketBeingRead ) {
-            bytesAvail = iBytesExpectedInPacketBeingRead;
+        if ( iBytesExpectedInPacketBeingRead == 0 &&
+                bytesAvail >= (quint64) sizeof(quint32) ) {
+            // we're not reading a packet yet, read packet
+            quint32 bytesExpectedNetworkByteOrder = 0 ;
+            if ( iSocket->read((char *)&bytesExpectedNetworkByteOrder,
+                               sizeof(quint32)) != sizeof(quint32)) {
+                iNeedsToRun = false ;
+                iModel.lock() ;
+                iModel.connectionStateChanged(Connection::Closing, this) ;
+                iModel.unlock() ;
+                iSocket->abort() ;
+                return ;
+            } else {
+                iBytesIn += sizeof(quint32) ;
+                bytesAvail = iSocket->bytesAvailable() ;
+            }
+            iBytesExpectedInPacketBeingRead = ntohl ( bytesExpectedNetworkByteOrder);
+            QLOG_STR(QString("0x%1 ").arg((qulonglong)this, 8) +"Connection::performRead expecting bytes= "+ QString::number( iBytesExpectedInPacketBeingRead)) ;
         }
-        QByteArray bytesReadThisTime ( iSocket->read(bytesAvail) ) ;
-        if ( bytesReadThisTime.size() > 0 ) {
-            iBytesRead->append(bytesReadThisTime) ;
-            iBytesIn += bytesReadThisTime.size() ;
+
+        if ( iBytesExpectedInPacketBeingRead > 0 &&
+                iBytesExpectedInPacketBeingRead > (quint64)(iBytesRead->size()) &&
+                bytesAvail > 0 ) {
+            if ( bytesAvail > iBytesExpectedInPacketBeingRead ) {
+                bytesAvail = ( iBytesExpectedInPacketBeingRead -
+                               iBytesRead->size() );
+            }
+            QByteArray bytesReadThisTime ( iSocket->read(bytesAvail) ) ;
+            if ( bytesReadThisTime.size() > 0 ) {
+                iBytesRead->append(bytesReadThisTime) ;
+                iBytesIn += bytesReadThisTime.size() ;
+                iInvocationsSinceLastByteReceived = 0 ;
+                // seems like we received something:
+                iTimeOfLastActivity = QDateTime::currentDateTimeUtc().toTime_t() ;
+            }
+        }
+        if ( iBytesExpectedInPacketBeingRead == (quint64)(iBytesRead->size()) ) {
+            // if we came here, we have one total packet..
+            if ( iObserver.dataReceived(*iBytesRead,
+                                        *this) == false ) {
+                QLOG_STR(QString("0x%1 ").arg((qulonglong)this, 8) +"### Closing due to unparseable data, peer = " + peerAddress().toString()) ;
+                iSocket->abort() ;
+                iNeedsToRun = false ;
+            } else {
+                QCoreApplication::processEvents(QEventLoop::ExcludeUserInputEvents) ;
+            }
+            iBytesExpectedInPacketBeingRead = 0 ;
+            iBytesRead->clear() ;
             iInvocationsSinceLastByteReceived = 0 ;
-            // seems like we received something:
-            iTimeOfLastActivity = QDateTime::currentDateTimeUtc().toTime_t() ;
+            iNumberOfPacketsReceived++ ;
         }
     }
-    if ( iBytesExpectedInPacketBeingRead == (quint64)(iBytesRead->size()) ) {
-        // if we came here, we have one total packet..
-        if ( iObserver.dataReceived(*iBytesRead,
-                                    *this) == false ) {
-            QLOG_STR(QString("0x%1 ").arg((qulonglong)this, 8) +"### Closing due to unparseable data, peer = " + peerAddress().toString()) ;
-            iSocket->disconnectFromHost() ;
-            iNeedsToRun = false ;
-        }
-        iBytesExpectedInPacketBeingRead = 0 ;
-        iBytesRead->clear() ;
-        iInvocationsSinceLastByteReceived = 0 ;
-        iNumberOfPacketsReceived++ ;
-    }
 }
 
 bool Connection::Ipv6AddressesEqual(const Q_IPV6ADDR& aAddr1,
@@ -403,101 +444,106 @@ bool Connection::Ipv6AddressesEqual(const Q_IPV6ADDR& aAddr1,
     return true ;
 }
 
+// practically this is more like write-loop because actual
+// reading and writing is done in SLOT()s that are connected
+// to read+write events.
 void Connection::readLoop() {
+    connect(iSocket, SIGNAL(readyRead ()),
+            this, SLOT(readyRead ()));
     iObserver.connectionReady(this) ; // before starting to read, inform the observer
     QByteArray *itemToSend = NULL ;
     while ( iNeedsToRun && iSocket->state() == QAbstractSocket::ConnectedState ) {
-        if ( iSocket->waitForReadyRead(500) == true ) {
-            if ( iNeedsToRun ) {
-                performRead() ;
-            }
+        iModel.lock() ;
+        checkForBucketFill() ;
+        int numberOfQueueItems = iSendQueue.size() ;
+        iModel.unlock() ;
+
+        if ( numberOfQueueItems > 0 ) {
+            msleep( 10 ) ;
         } else {
-            QCoreApplication::processEvents(QEventLoop::ExcludeUserInputEvents) ;
-            iModel.lock() ;
-            checkForBucketFill() ;
-            iModel.unlock() ;
-            // we came here because we have no bytes to read from
-            // sock. first check if we do have read operation pending
-            if ( iBytesExpectedInPacketBeingRead > 0 &&
-                    iBytesExpectedInPacketBeingRead > (quint64)(iBytesRead->size())) {
-                iInvocationsSinceLastByteReceived++ ; // gets incremented every 500ms
+            msleep( iSleepBetweenSendOperations ) ;
+        }
+        QCoreApplication::processEvents(QEventLoop::ExcludeUserInputEvents) ;
+
+        // we came here because we have no bytes to read from
+        // sock. first check if we do have read operation pending
+        if ( iBytesExpectedInPacketBeingRead > 0 &&
+                iBytesExpectedInPacketBeingRead > (quint64)(iBytesRead->size())) {
+            iInvocationsSinceLastByteReceived++ ; // gets incremented every 500ms
+        }
+        if ( iInvocationsSinceLastByteReceived > 1000 ) {
+            // we have had read operation pending for 500 seconds and
+            // not a single byte has got through -> give up
+            if ( iNeedsToRun ) {
+                iModel.lock() ;
+                LOG_STR(QString("0x%1 ").arg((qulonglong)this, 8) +"Read operation was pending too long, closing")  ;
+                iConnectionState = Connection::Closing ;
+                iModel.connectionStateChanged(Connection::Closing, this) ;
+                iModel.unlock() ;
+                iSocket->abort() ;
+                iNeedsToRun = false ;
+                LOG_STR(QString("0x%1 ").arg((qulonglong)this, 8) +"Giving up pending read operation after 500 seconds");
             }
-            if ( iInvocationsSinceLastByteReceived > 1000 ) {
-                // we have had read operation pending for 500 seconds and
-                // not a single byte has got through -> give up
+            break ;
+        }
+        // ok, then try write operation, if applicable
+        if ( iNeedsToRun && iSocket->bytesToWrite() == 0 ) {
+            iModel.lock() ;
+            itemToSend = iModel.nodeModel().getNextItemToSend(*this) ;
+            if ( itemToSend != NULL ) {
+                QLOG_STR("Got " + QString::number(itemToSend->size()) +
+                         " bytes to send, items in q = " +
+                         QString::number(iSendQueue.size())) ;
+                iSleepBetweenSendOperations = 30 ; // we got item to send, next time sleep only 30ms
+            } else {
+                iSleepBetweenSendOperations = 500 ; // queue empty, sleep 500 ms
+            }
+            iModel.unlock() ;
+        }
+        // now we own itemToSend
+        if ( itemToSend != NULL &&
+                iSocket != NULL &&
+                iSocket->state() == QAbstractSocket::ConnectedState ) {
+            quint32 bytesInPayload = itemToSend->size() ;
+            quint32 bytesInPayloadNetworkOrder = htonl(bytesInPayload) ;
+            QByteArray messageSizeBytes ;
+            messageSizeBytes.append((const char *)&bytesInPayloadNetworkOrder,
+                                    sizeof(quint32)) ;
+            if ( iSocket->write(messageSizeBytes) != sizeof(quint32) ) {
+                LOG_STR(QString("0x%1 ").arg((qulonglong)this, 8) +"Write operation did not succeed, closing..")  ;
                 if ( iNeedsToRun ) {
                     iModel.lock() ;
-                    LOG_STR(QString("0x%1 ").arg((qulonglong)this, 8) +"Read operation was pending too long, closing")  ;
                     iConnectionState = Connection::Closing ;
                     iModel.connectionStateChanged(Connection::Closing, this) ;
                     iModel.unlock() ;
-                    iSocket->abort() ;
-                    iNeedsToRun = false ;
-                    LOG_STR(QString("0x%1 ").arg((qulonglong)this, 8) +"Giving up pending read operation after 500 seconds");
                 }
-                break ;
+                iSocket->abort() ;
+                iNeedsToRun = false ;
+            } else {
+                iBytesOut += sizeof(quint32) ;
             }
-            // ok, then try write operation, if applicable
-            if ( iNeedsToRun ) {
-                iModel.lock() ;
-                itemToSend = iModel.nodeModel().getNextItemToSend(*this) ;
-                iModel.unlock() ;
-            }
-            // now we own itemToSend
-            if ( itemToSend != NULL &&
-                    iSocket != NULL &&
-                    iSocket->state() == QAbstractSocket::ConnectedState ) {
-                quint32 bytesInPayload = itemToSend->size() ;
-                quint32 bytesInPayloadNetworkOrder = htonl(bytesInPayload) ;
-                QByteArray messageSizeBytes ;
-                messageSizeBytes.append((const char *)&bytesInPayloadNetworkOrder,
-                                        sizeof(quint32)) ;
-                if ( iSocket->write(messageSizeBytes) != sizeof(quint32) ) {
-                    LOG_STR(QString("0x%1 ").arg((qulonglong)this, 8) +"Write operation did not succeed, closing..")  ;
-                    if ( iNeedsToRun ) {
-                        iModel.lock() ;
-                        iConnectionState = Connection::Closing ;
-                        iModel.connectionStateChanged(Connection::Closing, this) ;
-                        iModel.unlock() ;
-                    }
-                    iSocket->abort() ;
-                    iNeedsToRun = false ;
-                } else {
-                    iBytesOut += sizeof(quint32) ;
+            if ( iSocket->write(*itemToSend) != itemToSend->size() ) {
+                LOG_STR(QString("0x%1 ").arg((qulonglong)this, 8) +"Write operation did not succeed, closing..")  ;
+                if ( iNeedsToRun ) {
+                    iModel.lock() ;
+                    iConnectionState = Connection::Closing ;
+                    iModel.connectionStateChanged(Connection::Closing, this) ;
+                    iModel.unlock() ;
                 }
-                if ( iSocket->write(*itemToSend) != itemToSend->size() ) {
-                    LOG_STR(QString("0x%1 ").arg((qulonglong)this, 8) +"Write operation did not succeed, closing..")  ;
-                    if ( iNeedsToRun ) {
-                        iModel.lock() ;
-                        iConnectionState = Connection::Closing ;
-                        iModel.connectionStateChanged(Connection::Closing, this) ;
-                        iModel.unlock() ;
-                    }
-                    iSocket->abort() ;
-                    iNeedsToRun = false ;
-                } else {
-                    iBytesOut += itemToSend->size() ;
-                }
-                // ok ; how long to wait for write.
-                // very slow network here. have 100 connected clients.
-                // say, upload rate is 20kb/s, divided by 100 clients
-                // we get 204 bytes per sec. .. but at least 30 seconds.
-                if ( iSocket->waitForBytesWritten(((itemToSend->size() / 204) * 1000) + (30 * 1000)) == false ) {
-                    LOG_STR(QString("0x%1 ").arg((qulonglong)this, 8) +"Write wait operation did not succeed, closing..")  ;
-                    if ( iNeedsToRun ) {
-                        iModel.lock() ;
-                        iConnectionState = Connection::Closing ;
-                        iModel.connectionStateChanged(Connection::Closing, this) ;
-                        iModel.unlock() ;
-                    }
-                    iSocket->abort() ;
-                    iNeedsToRun = false ;
-                }
-                delete itemToSend ;
-                itemToSend = NULL ;
-                // seems like we sent something:
-                iTimeOfLastActivity = QDateTime::currentDateTimeUtc().toTime_t() ;
+                iSocket->abort() ;
+                iNeedsToRun = false ;
+            } else {
+                iBytesOut += itemToSend->size() ;
             }
+            // ok ; how long to wait for write.
+            // very slow network here. have 100 connected clients.
+            // say, upload rate is 20kb/s, divided by 100 clients
+            // we get 204 bytes per sec. .. but at least 30 seconds.
+            delete itemToSend ;
+            itemToSend = NULL ;
+            // seems like we sent something:
+            iTimeOfLastActivity = QDateTime::currentDateTimeUtc().toTime_t() ;
+            flushSocket() ;
         }
     }
     if ( iNeedsToRun && iSocket ) {
@@ -622,13 +668,17 @@ void Connection::checkForBucketFill() {
 }
 
 Hash Connection::getPeerHash() const {
-    if ( iConnectionState == Connection::Open && iSocket ) {
-        return Hash(   iSocket->peerCertificate() ) ;
+    if ( iPeerHash != KNullHash ) {
+        return iPeerHash ;
     } else {
-        // this may be KNullHash but for some outgoing connections
-        // this is anyway known already when socket is not
-        // yet in connected state.
-        return iFpOfNodeWeTrying ;
+        if ( iConnectionState == Connection::Open && iSocket ) {
+            return Hash(   iSocket->peerCertificate() ) ;
+        } else {
+            // this may be KNullHash but for some outgoing connections
+            // this is anyway known already when socket is not
+            // yet in connected state.
+            return iFpOfNodeWeTrying ;
+        }
     }
 }
 
@@ -637,3 +687,119 @@ bool Connection::forciblyCloseSocket() {
     iFpOfNodeWeTrying = KNullHash ;
     return true ;
 }
+
+
+void Connection::flushSocket() {
+    // setting TCP_NODELAY has side-effect of flushing the socket.
+    // at least in linux.
+    if ( iSocket ) {
+        iSocket->flush() ;
+        QCoreApplication::processEvents(QEventLoop::ExcludeUserInputEvents) ;
+        iSocket->setSocketOption(QAbstractSocket::LowDelayOption,0) ;
+        iSocket->setSocketOption(QAbstractSocket::LowDelayOption,1) ;
+        QCoreApplication::processEvents(QEventLoop::ExcludeUserInputEvents) ;
+    }
+}
+
+
+// Connected from QSslSocket. Called when bytes have been written
+void Connection::encryptedBytesWritten ( qint64
+#ifdef DEBUG
+        written
+#endif
+                                       ) {
+    QLOG_STR("Connection::encryptedBytesWritten " + QString::number(written));
+}
+
+// Connected from QSslSocket. Called on handshake errors
+void Connection::sslErrors ( const QList<QSslError> & errors  ) {
+    QLOG_STR("Connection::sslErrors") ;
+    foreach ( const QSslError& e , errors ) {
+        switch ( e.error() ) {
+        case QSslError::NoError:
+        case QSslError::CertificateExpired:
+        case QSslError::InvalidNotBeforeField:
+        case QSslError::InvalidNotAfterField:
+        case QSslError::SelfSignedCertificate:
+        case QSslError::SelfSignedCertificateInChain:
+        case QSslError::UnableToVerifyFirstCertificate:
+        case QSslError::CertificateRevoked:
+        case QSslError::InvalidCaCertificate:
+        case QSslError::CertificateUntrusted:
+        case QSslError::SubjectIssuerMismatch:
+        case QSslError::AuthorityIssuerSerialNumberMismatch:
+        case QSslError::HostNameMismatch:
+        case QSslError::InvalidPurpose:
+            QLOG_STR("Ssl error ignored: " + e.errorString()) ;
+            break ;
+        case QSslError::UnableToGetIssuerCertificate:
+        case QSslError::UnableToDecryptCertificateSignature:
+        case QSslError::UnableToDecodeIssuerPublicKey:
+        case QSslError::CertificateSignatureFailed:
+        case QSslError::CertificateNotYetValid:
+        case QSslError::UnableToGetLocalIssuerCertificate:
+        case QSslError::PathLengthExceeded:
+        case QSslError::CertificateRejected:
+        case QSslError::NoPeerCertificate:
+        case QSslError::UnspecifiedError:
+        case QSslError::NoSslSupport:
+        case QSslError::CertificateBlacklisted:
+            QLOG_STR("Ssl error detected: " + e.errorString()) ;
+            iNeedsToRun = false ;
+            break ;
+        }
+    }
+}
+// Connected from QSslSocket::error. Called on error
+void Connection::socketError ( QAbstractSocket::SocketError
+#ifdef DEBUG
+                               socketError
+#endif
+                             ) {
+    iNeedsToRun = false ;
+    QLOG_STR(QString("0x%1 ").arg((qulonglong)this, 8) +
+             "Connection::socketError "+ QString::number(socketError)) ;
+
+    if ( iSocketIsIncoming == false && // this is outgoing
+            iFpOfNodeWeTrying != KNullHash && // and we know who we tried to reach
+            iConnectionState == Connection::Initial // and didn't get past initial stage
+       ) {
+        emit connectionAttemptFailed(iFpOfNodeWeTrying) ; // then don't try again immediately
+
+    } else {
+        QLOG_STR("No emit") ;
+    }
+}
+
+// Connected from QSslSocket. Called on close
+void Connection::disconnected () {
+    iNeedsToRun = false ;
+    QLOG_STR(QString("0x%1 ").arg((qulonglong)this, 8) +"disconnected") ;
+}
+// Connected from QSslSocket. Called when bytes are available
+void Connection::readyRead () {
+    performRead() ;
+}
+
+void Connection::setupSocketSignals() {
+    connect(iSocket, SIGNAL(encrypted()), this, SLOT(socketReady()));
+    connect(iSocket, SIGNAL(encryptedBytesWritten ( qint64 )),
+            this, SLOT(encryptedBytesWritten ( qint64 )));
+    connect(iSocket, SIGNAL(sslErrors ( const QList<QSslError> & )),
+            this, SLOT(sslErrors ( const QList<QSslError> & )));
+    connect(iSocket, SIGNAL(error ( QAbstractSocket::SocketError )),
+            this, SLOT(socketError ( QAbstractSocket::SocketError )));
+    connect(iSocket, SIGNAL(disconnected ()),
+            this, SLOT(disconnected ()));
+    // note that readyread signal is connected when
+    // readloop is entered
+}
+
+void Connection::msleep(int aMilliSeconds) {
+    QWaitCondition waitCondition;
+    QMutex mutex;
+    QThread::yieldCurrentThread ();
+    mutex.lock() ; // waitCondition needs a mutex initially locked
+    waitCondition.wait(&mutex, aMilliSeconds);// give other threads a chance..
+    mutex.unlock() ;
+}
